#!/usr/bin/env python

# Inspire by Conbagui tutorial found here:https://www.deviantart.com/conbagui/art/Create-a-Tree-Bark-Text-effect-with-Gimp-2-10-793020774
# author: Pocholo
# date: 6/6/20

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugin folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.

#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.

from gimpfu import *
import gettext
import os, sys,time 

gettext.install("gimp20", gimp.locale_directory, unicode=True)

def create_tree_bark_text_effect(filename, customtext, fontname, size):
		
	#Initiates the temporary state
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()	
	
	#Open and image
    img = pdb.gimp_image_new(1920, 1080, RGB)
    imagLayer = pdb.gimp_file_load_layer(img, filename)
    pdb.gimp_image_add_layer(img, imagLayer, 0)   
    pdb.gimp_layer_resize_to_image_size(imagLayer)
    pdb.gimp_image_set_active_layer(img, imagLayer)
    pdb.gimp_item_set_name(imagLayer, "Background")
    
    #make image blurry
    pdb.plug_in_gauss(img, imagLayer, 60, 60, 1)   
	
    #Image process
    gimp.Display(img)  
	
	#Create the frame
    frameLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Frame", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, frameLayer, 0)
    pdb.gimp_layer_add_alpha(frameLayer)
    pdb.gimp_drawable_fill(frameLayer, FILL_TRANSPARENT)
    pdb.gimp_image_select_rectangle(img, 2, 100, 100, 1720, 880)
    pdb.gimp_selection_grow(img, 12)
    pdb.script_fu_distress_selection(img, frameLayer, 
	                                             127,     #Threshold 
												   8,     #Spread
												   8,     #Granularity  
												   1,     #Smooth
											   FALSE,	  #Smooth horizontally
											   FALSE)	  #Smooth _vertically
    pdb.gimp_selection_invert(img)
    pdb.gimp_context_set_foreground('White')
    pdb.gimp_edit_bucket_fill(frameLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)	
    pdb.gimp_selection_none(img)
	
	#Create the text		
    textW = size/0.4
    textH = size/1.4
    pdb.gimp_context_set_foreground((255, 255, 255))
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 10, 1, 600, PIXELS, fontname)
    pdb.gimp_text_layer_set_justification(textLayer, TEXT_JUSTIFY_CENTER)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, 18)
	
	#Resize text
    pdb.gimp_layer_scale (textLayer, textW, textH, TRUE)
			
	#Making the text centered	
    textLayer = pdb.gimp_image_get_active_layer(img)
    pdb.gimp_image_set_active_layer(img, textLayer)
    wh = pdb.gimp_drawable_width(textLayer)
    hh = pdb.gimp_drawable_height(textLayer)
    w = pdb.gimp_image_width(img)
    h = pdb.gimp_image_height(img)
    pdb.gimp_image_set_active_layer(img,textLayer)
    centerx=w/2
    centery=h/2
    adw=wh/2
    adh=hh/2
    sw=centerx - adw
    sh=centery - adh		
    pdb.gimp_layer_set_offsets(textLayer, sw, sh)
    pdb.gimp_layer_resize_to_image_size(textLayer)
	
	#Create branches
    BranchesLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Branches", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, BranchesLayer, 2)
    pdb.gimp_layer_add_alpha(BranchesLayer)
    pdb.gimp_drawable_fill(BranchesLayer, FILL_TRANSPARENT)
    pdb.gimp_image_select_polygon(img, 2, 70, [270, 395, 76, 60, 
                                                            415, 
                                                             58, #6
                                                            400, 
                                                            384, #8
                                                            571, 
                                                            445, #10
                                                            622, 
                                                             60, #12
                                                            780, 
                                                            430, #14
                                                           1018, 
                                                             60, #16
                                                           1090, 
                                                            492, #18
                                                           1348,  
                                                             60, #20
                                                           1552, 
                                                            368, #22
                                                           1810, 
                                                             40, #24
                                                           1872, 
                                                            470, #26
                                                           1048, 
                                                            493, #28
                                                           1872,
                                                            762, #30
                                                           1617,
                                                            675, #32
                                                           1662, 
                                                           1027, #34
                                                           1546,
                                                            675, #36
                                                           1125,
                                                            660, #38
                                                           1215, 
                                                           1027, #40
                                                            837,
                                                            633, #42
                                                            780,
                                                           1027, #44
                                                            750,
                                                            675, #46
                                                            580,
                                                            622, #48
                                                            435,
                                                           1027, #50
                                                            373,
                                                            528, #52
                                                            155,
                                                           1027, #54
                                                             60,
                                                            939, #56
                                                            270,
                                                            665, #58
                                                            265,
                                                            673, #60
                                                             60,
                                                            570, #62
                                                            262,
                                                            511, #64
                                                            260,
                                                            525, #66
                                                             60,
                                                            406, #68                                                                  
                                                            270,
                                                           395]) #70
    pdb.plug_in_sel2path(img, BranchesLayer) 
    pdb.gimp_selection_none(img)
    pdb.gimp_image_select_polygon(img, 2, 12, [1462, 40, 1269, 501, 1338, 1027, 1873, 1015, 1873, 40, 1462, 40])
    pdb.plug_in_sel2path(img, BranchesLayer) 
    pdb.gimp_selection_none(img)
    
    #Stroke both paths
    vectors1 = pdb.gimp_image_get_vectors_by_name(img, "Selection") 
    pdb.gimp_context_set_foreground("White")
    pdb.gimp_brushes_set_brush("2. Hardness 100")
    pdb.gimp_context_set_brush_size(20)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(0.1)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_stroke_method (STROKE_PAINT_METHOD)
    vectors1 = pdb.gimp_image_get_vectors_by_name(img, _("Selection")) 
    pdb.gimp_drawable_edit_stroke_item(BranchesLayer, vectors1)  
    vectors2 = pdb.gimp_image_get_vectors_by_name(img, "Selection #1") 
    pdb.gimp_context_set_foreground("White")
    pdb.gimp_brushes_set_brush("2. Hardness 100")
    pdb.gimp_context_set_brush_size(20)
    pdb.gimp_context_set_brush_angle(0)
    pdb.gimp_context_set_brush_spacing(0.1)
    pdb.gimp_context_set_brush_default_hardness()
    pdb.gimp_context_set_stroke_method (STROKE_PAINT_METHOD)
    vectors2 = pdb.gimp_image_get_vectors_by_name(img, (_("Selection")+' #1'))   
    pdb.gimp_drawable_edit_stroke_item(BranchesLayer, vectors2)
    
    #Make the branches wavy
    pdb.plug_in_ripple(img, BranchesLayer, 200, 25, 0, 1, 1, TRUE, TRUE)
    pdb.plug_in_ripple(img, BranchesLayer, 200, 25, 1, 1, 1, TRUE, TRUE)
    pdb.gimp_item_set_visible(imagLayer, FALSE)
    barkLayer = pdb.gimp_image_merge_visible_layers(img, 1)
    pdb.gimp_item_set_visible(imagLayer, TRUE)
    barkLayer= pdb.gimp_image_get_layer_by_name(img, "Branches")
    pdb.gimp_item_set_name(barkLayer, "Bark Text")
    
    #3d extrusion
    pdb.script_fu_3d_extrude(img, barkLayer, 
                                           0,  #extrusion type 
                                          10,  #extrusion depth 
                                         270,  #extrusion direction 
                                         145,  #bump azimuth 
                                          45,  #bump elevation 
                                           1,  #bump depth 
                                        0.30,  #bump ambient light 
                                           0,  #bump curve 
                                       FALSE,  #invert bump 
                                        TRUE,  #merge extrusion 
                                       FALSE,  #overlay with pattern 
                                       FALSE,  #extrude with pattern 
                                 "Dried mud")  #extrision pattern)
    extrusion = pdb.gimp_image_get_layer_by_name(img, "Extrusion")
    pdb.gimp_item_set_name(extrusion, "Bark Text")
    pdb.gimp_item_set_visible(extrusion, FALSE)
      
    #Inner shadow
    pdb.python_layerfx_inner_shadow(img, extrusion, 
                                         ("Black"),   #color
                                                75,   #opacity
                                                 0,   #contour - linear
                                                 0,   #noise
                                                 3,   #mode - multiply
                                                 1,   #source
                                                14,   #choke 
                                                10,   #size 
                                                -45,   #offset_angle 
                                                12,   #offset_distance 
                                             FALSE)  #merge
    
	#Inner Glow                           
    pdb.python_layerfx_inner_glow(img, extrusion,
								 (203, 190, 100),   #color
								              85,   #opacity
								               0,   #contou - linear
								               0,   #noise
                                              16,   #mode - dodge
                                               1,   #source
                                              10,   #choke
                                              10,   #size
                                           FALSE)   #merge
	
	#Bevel and emboss
    pdb.python_layerfx_bevel_emboss(img, extrusion, 
												 1,   #style inner bevel
											    65,   #depth 
												 0,   #direction 
											    20,   #size 
											     0,   #soften
											   135,   #angle 
											    35,   #altitude 
											     0,   #gloss_contour - linear
								   (248, 220, 152),   #highlight_color 
											     5,   #highlight_mode - overlay
											   100,   #highlight_opacity
										   "Black",   #shadow_color 
											     5,   #shadow_mode - overlay
											    75,   #shadow_opacity 
												 0,   #surface_contour - linear
											  TRUE,   #use_texture 
					 "mossy bark texture gray.jpg",   #pattern 
											   35,   #scale 
											   100,   #tex_depth 
											 FALSE,   #invert 
											 FALSE)   #merge	
	
	#Satin
    pdb.python_layerfx_satin(img, extrusion, 
                                  ("Black"),   #color
                                         70,   #opacity
                                          3,   #mode MULTIPLY
                                         20,   #offset_angle 
                                         10,   #offset_distance
                                         12,   #size 
                                          5,   #contour - gaussian
                                      FALSE,   #invert 
                                      FALSE)   #merge	
	
	#Pattern overlay		
    pdb.python_layerfx_pattern_overlay(img, extrusion, 
                             "Mossy bark texture.jpg",   #pattern
                                                  100,   #opacity
                                                    0,   #mode - normal
                                                   35,   #scale
                                                    0,   #interpolation
                                                FALSE)   #merge
		
    imagLayer = pdb.gimp_image_get_layer_by_name(img, "Background")
    pdb.gimp_image_set_active_layer(img, imagLayer)
        
    #Set gimp to default
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()
	
register(
    "create_tree_bark_text_effect",
    "Creates a Tree bark text",
    "Creates a Tree bark text text in a frame",
    "Pocholo",
    "Pocholo",
    "2020",
    "Create a Tree Bark Text",
    "",
[
	
    (PF_FILENAME, "filename", "Image:", None),
    (PF_STRING, "customtext", "Text string", "Forest"),
    (PF_FONT, "fontname", "Font name", "JFWildWood"),
    (PF_SPINNER, "size", "Text size", 600, (600, 600, 1)),	
    ],
    [], 
    create_tree_bark_text_effect, menu="<Image>/Pocholo-scripts/Create a Tree Bark Text",
    domain=("gimp20", gimp.locale_directory))
main()			
